package com.ruoyi.bst.store.service.impl;

import com.ruoyi.bst.store.domain.Store;
import com.ruoyi.bst.store.domain.StoreQuery;
import com.ruoyi.bst.store.domain.StoreVO;
import com.ruoyi.bst.store.domain.enums.StoreGroupBy;
import com.ruoyi.bst.store.domain.enums.StoreStatus;
import com.ruoyi.bst.store.mapper.StoreMapper;
import com.ruoyi.bst.store.service.IStoreService;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.vo.LongDecimalVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;

/**
 * 入库Service业务层处理
 *
 * @author ruoyi
 * @date 2024-12-19
 */
@Service
public class StoreServiceImpl implements IStoreService
{
    @Autowired
    private StoreMapper storeMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    /**
     * 查询入库
     *
     * @param id 入库主键
     * @return 入库
     */
    @Override
    public StoreVO selectStoreById(Long id)
    {
        return storeMapper.selectStoreById(id);
    }

    /**
     * 查询入库列表
     *
     * @param store 入库
     * @return 入库
     */
    @Override
    public List<StoreVO> selectStoreList(StoreQuery store)
    {
        return storeMapper.selectStoreList(store);
    }

    /**
     * 新增入库
     *
     * @param store 入库
     * @return 结果
     */
    @Override
    public int insertStore(Store store)
    {
        store.setCreateTime(DateUtils.getNowDate());
        return storeMapper.insertStore(store);
    }

    /**
     * 修改入库
     *
     * @param store 入库
     * @return 结果
     */
    @Override
    public int updateStore(Store store)
    {
        return storeMapper.updateStore(store);
    }

    /**
     * 批量删除入库
     *
     * @param ids 需要删除的入库主键
     * @return 结果
     */
    @Override
    public int deleteStoreByIds(Long[] ids)
    {
        return storeMapper.deleteStoreByIds(ids);
    }

    /**
     * 删除入库信息
     *
     * @param id 入库主键
     * @return 结果
     */
    @Override
    public int deleteStoreById(Long id)
    {
        return storeMapper.deleteStoreById(id);
    }

    @Override
    public int logicDel(Long id) {
        if (id == null) {
            return 0;
        }
        StoreVO old = selectStoreById(id);
        ServiceUtil.assertion(old == null, "待删除的数据不存在");
        ServiceUtil.assertion(!StoreStatus.canDel().contains(old.getStatus()), "当前数据状态不允许删除");

        Integer result = transactionTemplate.execute(status -> {
            Store data = new Store();
            data.setDeleted(true);
            StoreQuery query = new StoreQuery();
            query.setId(id);
            query.setStatusList(StoreStatus.canDel());
            int del = this.updateByQuery(data, query);
            ServiceUtil.assertion(del != 1, "删除失败");
            return del;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int stockIn(Long id, BigDecimal storeNum, SysUser user) {
        // 校验
        if (id == null || storeNum == null || storeNum.compareTo(BigDecimal.ZERO) <= 0) {
            return 0;
        }
        StoreVO old = selectStoreById(id);
        ServiceUtil.assertion(old == null, "待入库的单据不存在");
        ServiceUtil.assertion(!StoreStatus.canStockIn().contains(old.getStatus()), "当前数据状态不允许入库");

        // 更新数据
        Store data = new Store();
        data.setStoreNum(storeNum);
        if (user != null) {
            data.setStoreUserId(user.getUserId());
            data.setStoreBy(user.getNickName());
        }
        data.setStatus(StoreStatus.STORED.getStatus());
        data.setStoreTime(LocalDateTime.now());
        StoreQuery query = new StoreQuery();
        query.setId(id);
        query.setStatusList(StoreStatus.canStockIn());
        return storeMapper.updateByQuery(data, query);
    }

    @Override
    public List<LongDecimalVO> selectSumOfNumGroupByLong(StoreQuery query, StoreGroupBy groupBy) {
        if (groupBy == null) {
            return Collections.emptyList();
        }
        return storeMapper.selectSumOfNumGroupByLong(query, groupBy.getGroupBy());
    }

    @Override
    public List<LongDecimalVO> selectSumOfStoreNumGroupByLong(StoreQuery query, StoreGroupBy groupBy) {
        if (groupBy == null) {
            return Collections.emptyList();
        }
        return storeMapper.selectSumOfStoreNumGroupByLong(query, groupBy.getGroupBy());
    }

    private int updateByQuery(Store data, StoreQuery query) {
        return storeMapper.updateByQuery(data, query);
    }
}
